@@ -2,8 +2,11 @@
 #include <stdbool.h>
 #include <assert.h>
 
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "windef.h"
 #include "winbase.h"
+#include "winternl.h"
 #include "wine/debug.h"
 #include "wine/heap.h"
 
@@ -11,28 +14,50 @@
 #include "wine/asm.h"
 
 #define COBJMACROS
+#include "initguid.h"
 #include "d3d11.h"
 #include "d3d12.h"
 
-#include "initguid.h"
+#include "dxgi1_6.h"
 
 #include "dxvk_interfaces.h"
 
 #include "amd_ags.h"
 
+#include "unixlib.h"
+
 WINE_DEFAULT_DEBUG_CHANNEL(amd_ags);
 
+#define AMD_AGS_CALL(func, args) WINE_UNIX_CALL( unix_ ## func, args )
+
+static INIT_ONCE unix_init_once = INIT_ONCE_STATIC_INIT;
+static BOOL unix_lib_initialized;
+
+static BOOL WINAPI init_unix_lib_once( INIT_ONCE *once, void *param, void **context )
+{
+    unix_lib_initialized = !__wine_init_unix_call() && !AMD_AGS_CALL( init, NULL );
+    return TRUE;
+}
+
+static BOOL init_unix_lib(void)
+{
+    InitOnceExecuteOnce( &unix_init_once, init_unix_lib_once, NULL, NULL );
+    return unix_lib_initialized;
+}
+
+static const char driver_version[] = "23.19.02-230831a-396538C-AMD-Software-Adrenalin-Edition";
+static const char radeon_version[] = "23.10.2";
+
 enum amd_ags_version
 {
+    AMD_AGS_VERSION_5_0_5,
     AMD_AGS_VERSION_5_1_1,
     AMD_AGS_VERSION_5_2_0,
-    AMD_AGS_VERSION_5_2_1,
     AMD_AGS_VERSION_5_3_0,
     AMD_AGS_VERSION_5_4_0,
     AMD_AGS_VERSION_5_4_1,
     AMD_AGS_VERSION_5_4_2,
     AMD_AGS_VERSION_6_0_0,
-    AMD_AGS_VERSION_6_0_1,
     AMD_AGS_VERSION_6_1_0,
 
     AMD_AGS_VERSION_COUNT
@@ -40,44 +65,47 @@ enum amd_ags_version
 
 static const struct
 {
-    int major;
-    int minor;
-    int patch;
+    unsigned int ags_min_public_version;
+    unsigned int ags_max_public_version;
     unsigned int device_size;
     unsigned int dx11_returned_params_size;
+    int max_asicFamily;
 }
 amd_ags_info[AMD_AGS_VERSION_COUNT] =
 {
-    {5, 1, 1, sizeof(AGSDeviceInfo_511), sizeof(AGSDX11ReturnedParams_511)},
-    {5, 2, 0, sizeof(AGSDeviceInfo_520), sizeof(AGSDX11ReturnedParams_520)},
-    {5, 2, 1, sizeof(AGSDeviceInfo_520), sizeof(AGSDX11ReturnedParams_520)},
-    {5, 3, 0, sizeof(AGSDeviceInfo_520), sizeof(AGSDX11ReturnedParams_520)},
-    {5, 4, 0, sizeof(AGSDeviceInfo_540), sizeof(AGSDX11ReturnedParams_520)},
-    {5, 4, 1, sizeof(AGSDeviceInfo_541), sizeof(AGSDX11ReturnedParams_520)},
-    {5, 4, 2, sizeof(AGSDeviceInfo_542), sizeof(AGSDX11ReturnedParams_520)},
-    {6, 0, 0, sizeof(AGSDeviceInfo_600), sizeof(AGSDX11ReturnedParams_600)},
-    {6, 0, 1, sizeof(AGSDeviceInfo_600), sizeof(AGSDX11ReturnedParams_600)},
-    {6, 1, 0, sizeof(AGSDeviceInfo_600), sizeof(AGSDX11ReturnedParams_600)},
+    {AGS_MAKE_VERSION(5, 0, 0), AGS_MAKE_VERSION(5, 0, 6), sizeof(AGSDeviceInfo_511), sizeof(AGSDX11ReturnedParams_511), 0},
+    {AGS_MAKE_VERSION(5, 1, 1), AGS_MAKE_VERSION(5, 1, 1), sizeof(AGSDeviceInfo_511), sizeof(AGSDX11ReturnedParams_511), 0},
+    {AGS_MAKE_VERSION(5, 2, 0), AGS_MAKE_VERSION(5, 2, 1), sizeof(AGSDeviceInfo_520), sizeof(AGSDX11ReturnedParams_520), 0},
+    {AGS_MAKE_VERSION(5, 3, 0), AGS_MAKE_VERSION(5, 3, 0), sizeof(AGSDeviceInfo_520), sizeof(AGSDX11ReturnedParams_520), 0},
+    {AGS_MAKE_VERSION(5, 4, 0), AGS_MAKE_VERSION(5, 4, 0), sizeof(AGSDeviceInfo_540), sizeof(AGSDX11ReturnedParams_520), AsicFamily_RDNA},
+    {AGS_MAKE_VERSION(5, 4, 1), AGS_MAKE_VERSION(5, 4, 1), sizeof(AGSDeviceInfo_541), sizeof(AGSDX11ReturnedParams_520), AsicFamily_RDNA},
+    {AGS_MAKE_VERSION(5, 4, 2), AGS_MAKE_VERSION(5, 4, 2), sizeof(AGSDeviceInfo_542), sizeof(AGSDX11ReturnedParams_520), AsicFamily_RDNA},
+    {AGS_MAKE_VERSION(6, 0, 0), AGS_MAKE_VERSION(6, 0, 1), sizeof(AGSDeviceInfo_600), sizeof(AGSDX11ReturnedParams_600), AsicFamily_RDNA2},
+    {AGS_MAKE_VERSION(6, 1, 0), AGS_MAKE_VERSION(6, 2, 0), sizeof(AGSDeviceInfo_600), sizeof(AGSDX11ReturnedParams_600), AsicFamily_RDNA3},
 };
 
-#define DEF_FIELD(name) {DEVICE_FIELD_##name, {offsetof(AGSDeviceInfo_511, name), offsetof(AGSDeviceInfo_520, name), \
-        offsetof(AGSDeviceInfo_520, name), offsetof(AGSDeviceInfo_520, name), offsetof(AGSDeviceInfo_540, name), \
-        offsetof(AGSDeviceInfo_541, name), offsetof(AGSDeviceInfo_542, name), offsetof(AGSDeviceInfo_600, name), \
+#define DEF_FIELD(name) {DEVICE_FIELD_##name, {offsetof(AGSDeviceInfo_511, name), offsetof(AGSDeviceInfo_511, name), offsetof(AGSDeviceInfo_520, name), \
+        offsetof(AGSDeviceInfo_520, name), offsetof(AGSDeviceInfo_540, name), \
+        offsetof(AGSDeviceInfo_541, name), offsetof(AGSDeviceInfo_542, name), \
         offsetof(AGSDeviceInfo_600, name), offsetof(AGSDeviceInfo_600, name)}}
-#define DEF_FIELD_520_BELOW(name) {DEVICE_FIELD_##name, {offsetof(AGSDeviceInfo_511, name), offsetof(AGSDeviceInfo_520, name), \
-        offsetof(AGSDeviceInfo_520, name), offsetof(AGSDeviceInfo_520, name), -1, \
-        -1, -1, -1, -1, -1}}
-#define DEF_FIELD_540_UP(name) {DEVICE_FIELD_##name, {-1, -1, \
-        -1, -1, offsetof(AGSDeviceInfo_540, name), \
-        offsetof(AGSDeviceInfo_541, name), offsetof(AGSDeviceInfo_542, name), offsetof(AGSDeviceInfo_600, name), \
+#define DEF_FIELD_520_BELOW(name) {DEVICE_FIELD_##name, {offsetof(AGSDeviceInfo_511, name), offsetof(AGSDeviceInfo_511, name), offsetof(AGSDeviceInfo_520, name), \
+        offsetof(AGSDeviceInfo_520, name), -1, \
+        -1, -1, -1, -1}}
+#define DEF_FIELD_520_UP(name) {DEVICE_FIELD_##name, {-1, -1, offsetof(AGSDeviceInfo_520, name), \
+        offsetof(AGSDeviceInfo_520, name), offsetof(AGSDeviceInfo_540, name), \
+        offsetof(AGSDeviceInfo_541, name), offsetof(AGSDeviceInfo_542, name), \
         offsetof(AGSDeviceInfo_600, name), offsetof(AGSDeviceInfo_600, name)}}
-#define DEF_FIELD_540_600(name) {DEVICE_FIELD_##name, {-1, -1, \
-        -1, -1, offsetof(AGSDeviceInfo_540, name), \
+#define DEF_FIELD_540_UP(name) {DEVICE_FIELD_##name, {-1, -1, -1, \
+        -1, offsetof(AGSDeviceInfo_540, name), \
+        offsetof(AGSDeviceInfo_541, name), offsetof(AGSDeviceInfo_542, name), \
+        offsetof(AGSDeviceInfo_600, name), offsetof(AGSDeviceInfo_600, name)}}
+#define DEF_FIELD_540_600(name) {DEVICE_FIELD_##name, {-1, -1, -1, \
+        -1, offsetof(AGSDeviceInfo_540, name), \
+        offsetof(AGSDeviceInfo_541, name), offsetof(AGSDeviceInfo_542, name), \
+        -1, -1}}
+#define DEF_FIELD_600_BELOW(name) {DEVICE_FIELD_##name, {offsetof(AGSDeviceInfo_511, name), offsetof(AGSDeviceInfo_511, name), offsetof(AGSDeviceInfo_520, name), \
+        offsetof(AGSDeviceInfo_520, name), offsetof(AGSDeviceInfo_540, name), \
         offsetof(AGSDeviceInfo_541, name), offsetof(AGSDeviceInfo_542, name), \
-        -1, -1, -1}}
-#define DEF_FIELD_600_BELOW(name) {DEVICE_FIELD_##name, {offsetof(AGSDeviceInfo_511, name), offsetof(AGSDeviceInfo_520, name), \
-        offsetof(AGSDeviceInfo_520, name), offsetof(AGSDeviceInfo_520, name), offsetof(AGSDeviceInfo_540, name), \
-        offsetof(AGSDeviceInfo_541, name), offsetof(AGSDeviceInfo_542, name), -1, \
         -1, -1}}
 
 #define DEVICE_FIELD_adapterString 0
@@ -91,6 +119,14 @@ amd_ags_info[AMD_AGS_VERSION_COUNT] =
 #define DEVICE_FIELD_displays 8
 #define DEVICE_FIELD_isAPU 9
 
+#define DEVICE_FIELD_numCUs 10
+#define DEVICE_FIELD_coreClock 11
+#define DEVICE_FIELD_memoryClock 12
+#define DEVICE_FIELD_teraFlops 13
+#define DEVICE_FIELD_numWGPs 14
+#define DEVICE_FIELD_numROPs 15
+#define DEVICE_FIELD_memoryBandwidth 16
+
 static const struct
 {
     unsigned int field_index;
@@ -108,6 +144,13 @@ device_struct_fields[] =
     DEF_FIELD(numDisplays),
     DEF_FIELD(displays),
     DEF_FIELD_540_600(isAPU),
+    DEF_FIELD(numCUs),
+    DEF_FIELD(coreClock),
+    DEF_FIELD(memoryClock),
+    DEF_FIELD(teraFlops),
+    DEF_FIELD_540_UP(numWGPs),
+    DEF_FIELD_520_UP(numROPs),
+    DEF_FIELD_520_UP(memoryBandwidth),
 };
 
 #undef DEF_FIELD
@@ -131,6 +174,7 @@ struct AGSContext
     VkPhysicalDeviceMemoryProperties *memory_properties;
     ID3D11DeviceContext *d3d11_context;
     AGSDX11ExtensionsSupported_600 extensions;
+    unsigned int public_version;
 };
 
 static HMODULE hd3d11, hd3d12;
@@ -138,6 +182,15 @@ static typeof(D3D12CreateDevice) *pD3D12CreateDevice;
 static typeof(D3D11CreateDevice) *pD3D11CreateDevice;
 static typeof(D3D11CreateDeviceAndSwapChain) *pD3D11CreateDeviceAndSwapChain;
 
+#define AGS_VER_MAJOR(ver) ((ver) >> 22)
+#define AGS_VER_MINOR(ver) (((ver) >> 12) & ((1 << 10) - 1))
+#define AGS_VER_PATCH(ver) ((ver) & ((1 << 12) - 1))
+
+static const char *debugstr_agsversion(unsigned int ags_version)
+{
+    return wine_dbg_sprintf("%d.%d.%d", AGS_VER_MAJOR(ags_version), AGS_VER_MINOR(ags_version), AGS_VER_PATCH(ags_version));
+}
+
 static BOOL load_d3d12_functions(void)
 {
     if (hd3d12)
@@ -245,28 +298,105 @@ done:
     return ret;
 }
 
-static enum amd_ags_version determine_ags_version(void)
+static enum amd_ags_version get_version_number(int ags_version)
+{
+    unsigned int i;
+
+    for (i = 0; i < ARRAY_SIZE(amd_ags_info); i++)
+        if (ags_version >= amd_ags_info[i].ags_min_public_version && ags_version <= amd_ags_info[i].ags_max_public_version)
+        {
+            TRACE("Found AGS v%s (internal %d).\n", debugstr_agsversion(ags_version), i);
+            return i;
+        }
+    ERR("Unknown ags_version %s, using 5.4.1.\n", debugstr_agsversion(ags_version));
+    return AMD_AGS_VERSION_5_4_1;
+}
+
+static BOOL get_ags_version_from_resource(const WCHAR *filename, enum amd_ags_version *ret, int *public_version)
+{
+    DWORD infosize;
+    void *infobuf;
+    void *val;
+    UINT vallen;
+    VS_FIXEDFILEINFO *info;
+    UINT16 major, minor, patch;
+
+    infosize = GetFileVersionInfoSizeW(filename, NULL);
+    if (!infosize)
+    {
+        ERR("File version info not found, err %u.\n", GetLastError());
+        return FALSE;
+    }
+
+    if (!(infobuf = heap_alloc(infosize)))
+    {
+        ERR("Failed to allocate memory.\n");
+        return FALSE;
+    }
+
+    if (!GetFileVersionInfoW(filename, 0, infosize, infobuf))
+    {
+        ERR("GetFileVersionInfoW failed, err %u.\n", GetLastError());
+        heap_free(infobuf);
+        return FALSE;
+    }
+
+    if (!VerQueryValueW(infobuf, L"\\", &val, &vallen) || (vallen != sizeof(VS_FIXEDFILEINFO)))
+    {
+        ERR("Version value not found, err %u.\n", GetLastError());
+        heap_free(infobuf);
+        return FALSE;
+    }
+
+    info = val;
+    major = info->dwFileVersionMS >> 16;
+    minor = info->dwFileVersionMS;
+    patch = info->dwFileVersionLS >> 16;
+    *public_version = AGS_MAKE_VERSION(major, minor, patch);
+    TRACE("Found amd_ags_x64.dll v%d.%d.%d\n", major, minor, patch);
+    *ret = get_version_number(*public_version);
+    heap_free(infobuf);
+    return TRUE;
+}
+
+static enum amd_ags_version guess_version_from_exports(HMODULE hnative)
+{
+    /* Known DLL versions without version info:
+     *  - An update to AGS 5.4.1 included an amd_ags_x64.dll with no file version info;
+     *  - CoD: Modern Warfare Remastered (2017) ships dll without version info which is version 5.0.1
+     *    (not tagged in AGSSDK history), compatible with 5.0.5.
+     */
+    if (GetProcAddress(hnative, "agsDriverExtensionsDX11_Init"))
+    {
+        /* agsDriverExtensionsDX11_Init was deprecated in 5.3.0 */
+        TRACE("agsDriverExtensionsDX11_Init found.\n");
+        return AMD_AGS_VERSION_5_0_5;
+    }
+    TRACE("Returning 5.4.1.\n");
+    return AMD_AGS_VERSION_5_4_1;
+}
+
+static enum amd_ags_version determine_ags_version(int *ags_version)
 {
     /* AMD AGS is not binary compatible between versions (even minor versions), and the game
      * does not request a specific version when calling agsInit().
      * Checking the version of amd_ags_x64.dll shipped with the game is the only way to
      * determine what version the game was built against.
-     *
-     * An update to AGS 5.4.1 included an amd_ags_x64.dll with no file version info.
-     * In case of an error, assume it's that version.
      */
     enum amd_ags_version ret = AMD_AGS_VERSION_5_4_1;
-    DWORD infosize;
-    void *infobuf = NULL;
-    void *val;
-    UINT vallen, i;
-    VS_FIXEDFILEINFO *info;
-    UINT16 major, minor, patch;
     WCHAR dllname[MAX_PATH], temp_path[MAX_PATH], temp_name[MAX_PATH];
+    int (WINAPI *pagsGetVersionNumber)(void);
+    HMODULE hnative = NULL;
+    DWORD size;
+
+    TRACE("*ags_version %#x.\n", *ags_version);
+
+    if (*ags_version)
+        return get_version_number(*ags_version);
 
     *temp_name = 0;
-    if (!(infosize = GetModuleFileNameW(GetModuleHandleW(L"amd_ags_x64.dll"), dllname, ARRAY_SIZE(dllname)))
-            || infosize == ARRAY_SIZE(dllname))
+    if (!(size = GetModuleFileNameW(GetModuleHandleW(L"amd_ags_x64.dll"), dllname, ARRAY_SIZE(dllname)))
+            || size == ARRAY_SIZE(dllname))
     {
         ERR("GetModuleFileNameW failed.\n");
         goto done;
@@ -282,55 +412,36 @@ static enum amd_ags_version determine_ags_version(void)
         goto done;
     }
 
-    infosize = GetFileVersionInfoSizeW(temp_name, NULL);
-    if (!infosize)
+    if (get_ags_version_from_resource(temp_name, &ret, ags_version))
+        goto done;
+
+    if (!(hnative = LoadLibraryW(temp_name)))
     {
-        ERR("Unable to determine desired version of amd_ags_x64.dll.\n");
+        ERR("LoadLibraryW failed for %s.\n", debugstr_w(temp_name));
         goto done;
     }
 
-    if (!(infobuf = heap_alloc(infosize)))
+    if ((pagsGetVersionNumber = (void *)GetProcAddress(hnative, "agsGetVersionNumber")))
     {
-        ERR("Failed to allocate memory.\n");
+        *ags_version = pagsGetVersionNumber();
+        ret = get_version_number(*ags_version);
+        TRACE("Got version %s (%d) from agsGetVersionNumber.\n", debugstr_agsversion(*ags_version), ret);
         goto done;
     }
 
-    if (!GetFileVersionInfoW(temp_name, 0, infosize, infobuf))
-    {
-        ERR("Unable to determine desired version of amd_ags_x64.dll.\n");
-        goto done;
-    }
-
-    if (!VerQueryValueW(infobuf, L"\\", &val, &vallen) || (vallen != sizeof(VS_FIXEDFILEINFO)))
-    {
-        ERR("Unable to determine desired version of amd_ags_x64.dll.\n");
-        goto done;
-    }
-
-    info = val;
-    major = info->dwFileVersionMS >> 16;
-    minor = info->dwFileVersionMS;
-    patch = info->dwFileVersionLS >> 16;
-    TRACE("Found amd_ags_x64.dll v%d.%d.%d\n", major, minor, patch);
-
-    for (i = 0; i < ARRAY_SIZE(amd_ags_info); i++)
-    {
-        if ((major == amd_ags_info[i].major) &&
-            (minor == amd_ags_info[i].minor) &&
-            (patch == amd_ags_info[i].patch))
-        {
-            ret = i;
-            break;
-        }
-    }
+    ret = guess_version_from_exports(hnative);
 
 done:
+    if (!*ags_version)
+        *ags_version = amd_ags_info[ret].ags_max_public_version;
+
+    if (hnative)
+        FreeLibrary(hnative);
+
     if (*temp_name)
         DeleteFileW(temp_name);
 
-    heap_free(infobuf);
-    TRACE("Using AGS v%d.%d.%d interface\n",
-          amd_ags_info[ret].major, amd_ags_info[ret].minor, amd_ags_info[ret].patch);
+    TRACE("Using AGS v%s (internal %d) interface\n", debugstr_agsversion(*ags_version), ret);
     return ret;
 }
 
@@ -339,8 +450,103 @@ struct monitor_enum_context_600
     const char *adapter_name;
     AGSDisplayInfo_600 **ret_displays;
     int *ret_display_count;
+    IDXGIFactory1 *dxgi_factory;
 };
 
+static void create_dxgi_factory(HMODULE *hdxgi, IDXGIFactory1 **factory)
+{
+    typeof(CreateDXGIFactory1) *pCreateDXGIFactory1;
+
+    *factory = NULL;
+
+    if (!(*hdxgi = LoadLibraryW(L"dxgi.dll")))
+    {
+        ERR("Could not load dxgi.dll.\n");
+        return;
+    }
+
+    if (!(pCreateDXGIFactory1 = (void *)GetProcAddress(*hdxgi, "CreateDXGIFactory1")))
+    {
+        ERR("Could not find CreateDXGIFactory1.\n");
+        return;
+    }
+
+    if (FAILED(pCreateDXGIFactory1(&IID_IDXGIFactory1, (void**)factory)))
+        return;
+}
+
+static void release_dxgi_factory(HMODULE hdxgi, IDXGIFactory1 *factory)
+{
+    if (factory)
+        IDXGIFactory1_Release(factory);
+    if (hdxgi)
+        FreeLibrary(hdxgi);
+}
+
+static void fill_chroma_info(AGSDisplayInfo_600 *info, struct monitor_enum_context_600 *c, HMONITOR monitor)
+{
+    DXGI_OUTPUT_DESC1 output_desc;
+    IDXGIAdapter1 *adapter;
+    IDXGIOutput6 *output6;
+    IDXGIOutput *output;
+    BOOL found = FALSE;
+    unsigned int i, j;
+    HRESULT hr;
+
+    i = 0;
+    while (!found && (SUCCEEDED(IDXGIFactory1_EnumAdapters1(c->dxgi_factory, i++, &adapter))))
+    {
+        j = 0;
+        while (SUCCEEDED(IDXGIAdapter1_EnumOutputs(adapter, j++, &output)))
+        {
+            hr = IDXGIOutput_QueryInterface(output, &IID_IDXGIOutput6, (void**)&output6);
+            IDXGIOutput_Release(output);
+            if (FAILED(hr))
+            {
+                WARN("Failed to query IDXGIOutput6.\n");
+                continue;
+            }
+            hr = IDXGIOutput6_GetDesc1(output6, &output_desc);
+            IDXGIOutput6_Release(output6);
+
+            if (FAILED(hr) || output_desc.Monitor != monitor)
+                continue;
+            found = TRUE;
+
+            TRACE("output_desc.ColorSpace %#x.\n", output_desc.ColorSpace);
+            if (output_desc.ColorSpace == DXGI_COLOR_SPACE_RGB_FULL_G2084_NONE_P2020)
+            {
+                TRACE("Reporting monitor %s as HDR10 supported.\n", debugstr_a(info->displayDeviceName));
+                info->HDR10 = 1;
+            }
+
+            info->chromaticityRedX = output_desc.RedPrimary[0];
+            info->chromaticityRedY = output_desc.RedPrimary[1];
+            info->chromaticityGreenX = output_desc.GreenPrimary[0];
+            info->chromaticityGreenY = output_desc.GreenPrimary[1];
+            info->chromaticityBlueX = output_desc.BluePrimary[0];
+            info->chromaticityBlueY = output_desc.BluePrimary[1];
+            info->chromaticityWhitePointX = output_desc.WhitePoint[0];
+            info->chromaticityWhitePointY = output_desc.WhitePoint[1];
+
+            TRACE("chromacity: (%.6lf, %.6lf) (%.6lf, %.6lf) (%.6lf, %.6lf).\n", info->chromaticityRedX,
+                    info->chromaticityRedY, info->chromaticityGreenX, info->chromaticityGreenY, info->chromaticityBlueX,
+                    info->chromaticityBlueY);
+
+            info->screenDiffuseReflectance = 0;
+            info->screenSpecularReflectance = 0;
+
+            info->minLuminance = output_desc.MinLuminance;
+            info->maxLuminance = output_desc.MaxLuminance;
+            info->avgLuminance = output_desc.MaxFullFrameLuminance;
+        }
+        IDXGIAdapter1_Release(adapter);
+    }
+
+    if (!found)
+        WARN("dxgi output not found.\n");
+}
+
 static BOOL WINAPI monitor_enum_proc_600(HMONITOR hmonitor, HDC hdc, RECT *rect, LPARAM context)
 {
     struct monitor_enum_context_600 *c = (struct monitor_enum_context_600 *)context;
@@ -410,6 +616,9 @@ static BOOL WINAPI monitor_enum_proc_600(HMONITOR hmonitor, HDC hdc, RECT *rect,
             dev_mode.dmSize = sizeof(dev_mode);
         }
 
+        info->eyefinityGridCoordX = -1;
+        info->eyefinityGridCoordY = -1;
+
         info->currentResolution.offsetX = monitor_info.rcMonitor.left;
         info->currentResolution.offsetY = monitor_info.rcMonitor.top;
         info->currentResolution.width = monitor_info.rcMonitor.right - monitor_info.rcMonitor.left;
@@ -423,6 +632,9 @@ static BOOL WINAPI monitor_enum_proc_600(HMONITOR hmonitor, HDC hdc, RECT *rect,
             info->currentRefreshRate = dev_mode.dmDisplayFrequency;
         else
             ERR("Could not get current display settings.\n");
+
+        fill_chroma_info(info, c, hmonitor);
+
         ++*c->ret_display_count;
 
         TRACE("Added display %s for %s.\n", debugstr_a(monitor_info.szDevice), debugstr_a(c->adapter_name));
@@ -434,14 +646,17 @@ static BOOL WINAPI monitor_enum_proc_600(HMONITOR hmonitor, HDC hdc, RECT *rect,
 static void init_device_displays_600(const char *adapter_name, AGSDisplayInfo_600 **ret_displays, int *ret_display_count)
 {
     struct monitor_enum_context_600 context;
+    HMODULE hdxgi;
 
     TRACE("adapter_name %s.\n", debugstr_a(adapter_name));
 
     context.adapter_name = adapter_name;
     context.ret_displays = ret_displays;
     context.ret_display_count = ret_display_count;
+    create_dxgi_factory(&hdxgi, &context.dxgi_factory);
 
     EnumDisplayMonitors(NULL, NULL, monitor_enum_proc_600, (LPARAM)&context);
+    release_dxgi_factory(hdxgi, context.dxgi_factory);
 }
 
 static void init_device_displays_511(const char *adapter_name, AGSDisplayInfo_511 **ret_displays, int *ret_display_count)
@@ -467,7 +682,7 @@ static void init_device_displays_511(const char *adapter_name, AGSDisplayInfo_51
 }
 
 
-static AGSReturnCode init_ags_context(AGSContext *context)
+static AGSReturnCode init_ags_context(AGSContext *context, int ags_version)
 {
     AGSReturnCode ret;
     unsigned int i, j;
@@ -475,7 +690,8 @@ static AGSReturnCode init_ags_context(AGSContext *context)
 
     memset(context, 0, sizeof(*context));
 
-    context->version = determine_ags_version();
+    context->version = determine_ags_version(&ags_version);
+    context->public_version = ags_version;
 
     ret = vk_get_physical_device_properties(&context->device_count, &context->properties, &context->memory_properties);
     if (ret != AGS_SUCCESS || !context->device_count)
@@ -517,8 +733,28 @@ static AGSReturnCode init_ags_context(AGSContext *context)
         SET_DEVICE_FIELD(device, deviceId, int, context->version, vk_properties->deviceID);
         if (vk_properties->vendorID == 0x1002)
         {
+            struct get_device_info_params params =
+            {
+                .device_id = vk_properties->deviceID,
+            };
+
             SET_DEVICE_FIELD(device, architectureVersion, ArchitectureVersion, context->version, ArchitectureVersion_GCN);
-            SET_DEVICE_FIELD(device, asicFamily, AsicFamily, context->version, AsicFamily_GCN4);
+            if (init_unix_lib() && !AMD_AGS_CALL(get_device_info, &params))
+            {
+                SET_DEVICE_FIELD(device, asicFamily, AsicFamily, context->version,
+                        min(params.asic_family, amd_ags_info[context->version].max_asicFamily));
+                SET_DEVICE_FIELD(device, numCUs, int, context->version, params.num_cu);
+                SET_DEVICE_FIELD(device, numWGPs, int, context->version, params.num_wgp);
+                SET_DEVICE_FIELD(device, numROPs, int, context->version, params.num_rops);
+                SET_DEVICE_FIELD(device, coreClock, int, context->version, params.core_clock);
+                SET_DEVICE_FIELD(device, memoryClock, int, context->version, params.memory_clock);
+                SET_DEVICE_FIELD(device, memoryBandwidth, int, context->version, params.memory_bandwidth);
+                SET_DEVICE_FIELD(device, teraFlops, float, context->version, params.teraflops);
+            }
+            else
+            {
+                SET_DEVICE_FIELD(device, asicFamily, AsicFamily, context->version, AsicFamily_GCN4);
+            }
             if (vk_properties->deviceType == VK_PHYSICAL_DEVICE_TYPE_INTEGRATED_GPU)
             {
                 if (context->version >= AMD_AGS_VERSION_6_0_0)
@@ -576,18 +812,18 @@ AGSReturnCode WINAPI agsInit(AGSContext **context, const AGSConfiguration *confi
     if (!(object = heap_alloc(sizeof(*object))))
         return AGS_OUT_OF_MEMORY;
 
-    if ((ret = init_ags_context(object)) != AGS_SUCCESS)
+    if ((ret = init_ags_context(object, 0)) != AGS_SUCCESS)
     {
         heap_free(object);
         return ret;
     }
 
     memset(gpu_info, 0, sizeof(*gpu_info));
-    gpu_info->agsVersionMajor = amd_ags_info[object->version].major;
-    gpu_info->agsVersionMinor = amd_ags_info[object->version].minor;
-    gpu_info->agsVersionPatch = amd_ags_info[object->version].patch;
-    gpu_info->driverVersion = "21.30.25.05-211005a-372402E-RadeonSoftware";
-    gpu_info->radeonSoftwareVersion  = "21.10.2";
+    gpu_info->agsVersionMajor = AGS_VER_MAJOR(object->public_version);
+    gpu_info->agsVersionMinor = AGS_VER_MINOR(object->public_version);
+    gpu_info->agsVersionPatch = AGS_VER_PATCH(object->public_version);
+    gpu_info->driverVersion = driver_version;
+    gpu_info->radeonSoftwareVersion  = radeon_version;
     gpu_info->numDevices = object->device_count;
     gpu_info->devices = object->devices;
 
@@ -614,15 +850,15 @@ AGSReturnCode WINAPI agsInitialize(int ags_version, const AGSConfiguration *conf
     if (!(object = heap_alloc(sizeof(*object))))
         return AGS_OUT_OF_MEMORY;
 
-    if ((ret = init_ags_context(object)) != AGS_SUCCESS)
+    if ((ret = init_ags_context(object, ags_version)) != AGS_SUCCESS)
     {
         heap_free(object);
         return ret;
     }
 
     memset(gpu_info, 0, sizeof(*gpu_info));
-    gpu_info->driverVersion = "21.30.25.05-211005a-372402E-RadeonSoftware";
-    gpu_info->radeonSoftwareVersion  = "21.10.2";
+    gpu_info->driverVersion = driver_version;
+    gpu_info->radeonSoftwareVersion  = radeon_version;
     gpu_info->numDevices = object->device_count;
     gpu_info->devices = object->devices;
 
@@ -667,15 +903,108 @@ AGSReturnCode WINAPI agsDeInitialize(AGSContext *context)
     return AGS_SUCCESS;
 }
 
+static DXGI_COLOR_SPACE_TYPE convert_ags_colorspace_506(AGSDisplaySettings_Mode_506 mode)
+{
+    switch (mode)
+    {
+        default:
+            ERR("Unknown color space in AGS: %d.\n", mode);
+        /* fallthrough */
+        case Mode_506_SDR:
+            TRACE("Setting Mode_506_SDR.\n");
+            return DXGI_COLOR_SPACE_RGB_FULL_G22_NONE_P709;
+        case Mode_506_PQ:
+            TRACE("Setting Mode_506_PQ.\n");
+            return DXGI_COLOR_SPACE_RGB_FULL_G2084_NONE_P2020;
+        case Mode_506_scRGB:
+            TRACE("Setting Mode_506_scRGB.\n");
+            return DXGI_COLOR_SPACE_RGB_FULL_G10_NONE_P709;
+    }
+}
+
+static DXGI_COLOR_SPACE_TYPE convert_ags_colorspace_600(AGSDisplaySettings_Mode_600 mode)
+{
+    switch (mode)
+    {
+        default:
+            ERR("Unknown color space in AGS: %d\n", mode);
+        /* fallthrough */
+        case Mode_600_SDR:
+            TRACE("Setting Mode_600_SDR.\n");
+            return DXGI_COLOR_SPACE_RGB_FULL_G22_NONE_P709;
+        case Mode_600_HDR10_PQ:
+            TRACE("Setting Mode_600_HDR10_PQ.\n");
+            return DXGI_COLOR_SPACE_RGB_FULL_G2084_NONE_P2020;
+        case Mode_600_HDR10_scRGB:
+            TRACE("Setting Mode_600_HDR10_scRGB.\n");
+            return DXGI_COLOR_SPACE_RGB_FULL_G10_NONE_P709;
+    }
+}
+
+static DXGI_HDR_METADATA_HDR10 convert_ags_metadata(const AGSDisplaySettings_600 *settings)
+{
+    DXGI_HDR_METADATA_HDR10 metadata;
+    metadata.RedPrimary[0] = settings->chromaticityRedX * 50000;
+    metadata.RedPrimary[1] = settings->chromaticityRedY * 50000;
+    metadata.GreenPrimary[0] = settings->chromaticityGreenX * 50000;
+    metadata.GreenPrimary[1] = settings->chromaticityGreenY * 50000;
+    metadata.BluePrimary[0] = settings->chromaticityBlueX * 50000;
+    metadata.BluePrimary[1] = settings->chromaticityBlueY * 50000;
+    metadata.WhitePoint[0] = settings->chromaticityWhitePointX * 50000;
+    metadata.WhitePoint[1] = settings->chromaticityWhitePointY * 50000;
+    metadata.MaxMasteringLuminance = settings->maxLuminance;
+    metadata.MinMasteringLuminance = settings->minLuminance / 0.0001f;
+    metadata.MaxContentLightLevel = settings->maxContentLightLevel;
+    metadata.MaxFrameAverageLightLevel = settings->maxFrameAverageLightLevel;
+    return metadata;
+}
+
 AGSReturnCode WINAPI agsSetDisplayMode(AGSContext *context, int device_index, int display_index, const AGSDisplaySettings *settings)
 {
-    FIXME("context %p device_index %d display_index %d settings %p stub!\n", context, device_index,
+    const AGSDisplaySettings_506 *settings506 = &settings->agsDisplaySettings506;
+    const AGSDisplaySettings_600 *settings600 = &settings->agsDisplaySettings600;
+    IDXGIVkInteropFactory1 *dxgi_interop = NULL;
+    DXGI_COLOR_SPACE_TYPE colorspace;
+    DXGI_HDR_METADATA_HDR10 metadata;
+    AGSReturnCode ret = AGS_SUCCESS;
+    IDXGIFactory1 *dxgi_factory;
+    HMODULE hdxgi;
+
+    TRACE("context %p device_index %d display_index %d settings %p\n", context, device_index,
           display_index, settings);
 
     if (!context)
         return AGS_INVALID_ARGS;
 
-    return AGS_SUCCESS;
+    create_dxgi_factory(&hdxgi, &dxgi_factory);
+    if (!dxgi_factory)
+        goto done;
+
+    if (FAILED(IDXGIFactory1_QueryInterface(dxgi_factory, &IID_IDXGIVkInteropFactory1, (void**)&dxgi_interop)))
+    {
+        WARN("Failed to get IDXGIVkInteropFactory1.\n");
+        goto done;
+    }
+
+    colorspace = context->version < AMD_AGS_VERSION_5_1_1
+        ? convert_ags_colorspace_506(settings506->mode)
+        : convert_ags_colorspace_600(settings600->mode);
+    /* Settings 506, 511 and 600 are identical aside from enum order + use
+     * of bitfield flags we do not use. */
+    metadata = convert_ags_metadata(settings600);
+
+    TRACE("chromacity: (%.6lf, %.6lf) (%.6lf, %.6lf) (%.6lf, %.6lf).\n", settings600->chromaticityRedX,
+            settings600->chromaticityRedY, settings600->chromaticityGreenX, settings600->chromaticityGreenY,
+            settings600->chromaticityBlueX, settings600->chromaticityBlueY);
+
+    if (FAILED(IDXGIVkInteropFactory1_SetGlobalHDRState(dxgi_interop, colorspace, &metadata)))
+        ret = AGS_DX_FAILURE;
+
+done:
+    if (dxgi_interop)
+        IDXGIVkInteropFactory1_Release(dxgi_interop);
+    release_dxgi_factory(hdxgi, dxgi_factory);
+    return ret;
 }
 
 AGSReturnCode WINAPI agsGetCrossfireGPUCount(AGSContext *context, int *gpu_count)
@@ -837,18 +1166,19 @@ AGSReturnCode WINAPI agsDriverExtensionsDX12_DestroyDevice(AGSContext* context,
 
 AGSDriverVersionResult WINAPI agsCheckDriverVersion(const char* version_reported, unsigned int version_required)
 {
-    FIXME("version_reported %s, version_required %d semi-stub.\n", debugstr_a(version_reported), version_required);
+    WARN("version_reported %s, version_required %d semi-stub.\n", debugstr_a(version_reported), version_required);
 
     return AGS_SOFTWAREVERSIONCHECK_OK;
 }
 
 int WINAPI agsGetVersionNumber(void)
 {
-    enum amd_ags_version version = determine_ags_version();
+    int public_version = 0;
+    enum amd_ags_version version = determine_ags_version(&public_version);
 
-    TRACE("version %d.\n", version);
+    TRACE("version %s (internal %d).\n", debugstr_agsversion(public_version), version);
 
-    return AGS_MAKE_VERSION(amd_ags_info[version].major, amd_ags_info[version].minor, amd_ags_info[version].patch);
+    return public_version;
 }
 
 AGSReturnCode WINAPI agsDriverExtensionsDX11_Init( AGSContext *context, ID3D11Device *device, unsigned int uavSlot, unsigned int *extensionsSupported )
@@ -875,6 +1205,19 @@ AGSReturnCode WINAPI agsDriverExtensionsDX11_Init( AGSContext *context, ID3D11De
     return AGS_SUCCESS;
 }
 
+AGSReturnCode WINAPI agsDriverExtensionsDX11_DeInit( AGSContext* context )
+{
+    TRACE("context %p.\n", context);
+
+    if (context->d3d11_context)
+    {
+        ID3D11DeviceContext_Release(context->d3d11_context);
+        context->d3d11_context = NULL;
+    }
+
+    return AGS_SUCCESS;
+}
+
 BOOL WINAPI DllMain(HINSTANCE instance, DWORD reason, void *reserved)
 {
     TRACE("%p, %u, %p.\n", instance, reason, reserved);
@@ -938,6 +1281,7 @@ AGSReturnCode WINAPI agsDriverExtensionsDX11_SetDepthBounds_530(AGSContext* cont
     return set_depth_bounds(context, dx_context, enabled, min_depth, max_depth);
 }
 
+C_ASSERT(AMD_AGS_VERSION_5_3_0 == 3);
 __ASM_GLOBAL_FUNC( DX11_SetDepthBounds_impl,
                    "mov (%rcx),%eax\n\t" /* version */
                    "cmp $3,%eax\n\t"
@@ -989,6 +1333,7 @@ AGSReturnCode WINAPI agsDriverExtensionsDX11_BeginUAVOverlap(AGSContext *context
     return update_uav_overlap(context, dx_context, TRUE);
 }
 
+C_ASSERT(AMD_AGS_VERSION_5_3_0 == 3);
 __ASM_GLOBAL_FUNC( DX11_BeginUAVOverlap_impl,
                    "mov (%rcx),%eax\n\t" /* version */
                    "cmp $3,%eax\n\t"
@@ -1022,6 +1367,7 @@ AGSReturnCode WINAPI agsDriverExtensionsDX11_EndUAVOverlap(AGSContext *context,
     return update_uav_overlap(context, dx_context, FALSE);
 }
 
+C_ASSERT(AMD_AGS_VERSION_5_3_0 == 3);
 __ASM_GLOBAL_FUNC( DX11_EndUAVOverlap_impl,
                    "mov (%rcx),%eax\n\t" /* version */
                    "cmp $3,%eax\n\t"
@@ -1067,9 +1413,11 @@ AGSReturnCode WINAPI agsDriverExtensionsDX11_DestroyDevice_511(AGSContext *conte
 
     return agsDriverExtensionsDX11_DestroyDevice_520(context, device, references, NULL, NULL);
 }
+
+C_ASSERT(AMD_AGS_VERSION_5_2_0 == 2);
 __ASM_GLOBAL_FUNC( agsDriverExtensionsDX11_DestroyDevice,
                    "mov (%rcx),%eax\n\t" /* version */
-                   "cmp $1,%eax\n\t"
+                   "cmp $2,%eax\n\t"
                    "jge 1f\n\t"
                    "jmp "     __ASM_NAME("agsDriverExtensionsDX11_DestroyDevice_511") "\n\t"
                    "1:\tjmp " __ASM_NAME("agsDriverExtensionsDX11_DestroyDevice_520") )
