@@ -592,6 +592,9 @@ static const WCHAR *hack_append_command_line( const WCHAR *cmd )
         {L"DC Universe Online\\LaunchPad.exe", L" --use-gl=swiftshader"},
         {L"PlanetSide 2\\LaunchPad.exe", L" --use-gl=swiftshader"},
         {L"PaladinLias\\Game.exe", L" --use-gl=desktop"},
+        {L"EverQuest 2\\LaunchPad.exe", L" --use-gl=swiftshader"},
+        {L"Everquest F2P\\LaunchPad.exe", L" --use-gl=swiftshader"},
+        {L"Red Tie Runner.exe", L" --use-angle=gl"},
     };
     unsigned int i;
 
@@ -1925,6 +1928,16 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetEnvironmentVariableW( LPCWSTR name, LPCWSTR val
                 len = lstrlenW(names[i]);
                 if (size > len && !memcmp( module + size - len, names[i], len * sizeof(*module) ))
                 {
+                    HMODULE h = GetModuleHandleW(L"Qt5Core.dll");
+                    void (WINAPI *QCoreApplication_setAttribute)(int attr, BOOL set);
+
+                    QCoreApplication_setAttribute = (void *)GetProcAddress(h, "?setAttribute@QCoreApplication@@SAXW4ApplicationAttribute@Qt@@_N@Z");
+                    if (QCoreApplication_setAttribute)
+                    {
+                        QCoreApplication_setAttribute(16 /* AA_UseOpenGLES */, 0);
+                        QCoreApplication_setAttribute(15 /* AA_UseDesktopOpenGL */, 1);
+                    }
+                    else ERR("QCoreApplication_setAttribute not found, h %p.\n", h);
                     value = L"desktop";
                     FIXME( "HACK: setting QT_OPENGL=desktop.\n" );
                     break;
