@@ -62,6 +62,10 @@
 #include <net/if_types.h>
 #endif
 
+#ifdef HAVE_LINUX_WIRELESS_H
+#include <linux/wireless.h>
+#endif
+
 #include <pthread.h>
 
 #define NONAMELESSUNION
@@ -102,6 +106,8 @@ struct if_entry
 static struct list if_list = LIST_INIT( if_list );
 static pthread_mutex_t if_list_lock = PTHREAD_MUTEX_INITIALIZER;
 
+static BOOL have_ethernet_iface;
+
 static struct if_entry *find_entry_from_index( UINT index )
 {
     struct if_entry *entry;
@@ -173,6 +179,21 @@ static NTSTATUS if_get_physical( const char *name, UINT *type, IF_PHYSICAL_ADDRE
     if (*type == MIB_IF_TYPE_OTHER && !ioctl( fd, SIOCGIFFLAGS, &ifr ) && ifr.ifr_flags & IFF_POINTOPOINT)
         *type = MIB_IF_TYPE_PPP;
 
+#ifdef HAVE_LINUX_WIRELESS_H
+    if (*type == MIB_IF_TYPE_ETHERNET)
+    {
+        struct iwreq pwrq;
+
+        memset( &pwrq, 0, sizeof(pwrq) );
+        memcpy( pwrq.ifr_name, name, size );
+        if (ioctl( fd, SIOCGIWNAME, &pwrq ) != -1)
+        {
+            TRACE( "iface %s, wireless protocol %s.\n", debugstr_a(name), debugstr_a(pwrq.u.name) );
+            *type = IF_TYPE_IEEE80211;
+        }
+    }
+#endif
+
 err:
     close( fd );
     return ret;
@@ -257,7 +278,22 @@ static WCHAR *strdupAtoW( const char *str )
     return ret;
 }
 
-static struct if_entry *add_entry( UINT index, char *name )
+static int fake_ethernet_adapter(void)
+{
+    static int cached = -1;
+
+    if (cached == -1)
+    {
+        const char *s;
+        if ((s = getenv( "WINE_FAKE_ETH_PRESENCE" )))
+            cached = atoi( s );
+        else
+            cached = (s = getenv( "SteamGameId" )) && !strcmp( s, "1293830" );
+    }
+    return cached;
+}
+
+static struct if_entry *add_entry( UINT index, const char *name )
 {
     struct if_entry *entry;
     int name_len = strlen( name );
@@ -286,6 +322,10 @@ static struct if_entry *add_entry( UINT index, char *name )
     memcpy( entry->if_guid.Data4 + 2, "NetDev", 6 );
 
     list_add_tail( &if_list, &entry->entry );
+
+    if (entry->if_luid.Info.IfType == MIB_IF_TYPE_ETHERNET)
+        have_ethernet_iface = TRUE;
+
     return entry;
 }
 
@@ -293,6 +333,7 @@ static unsigned int update_if_table( void )
 {
     struct if_nameindex *indices = if_nameindex(), *entry;
     unsigned int append_count = 0;
+    struct if_entry *if_entry;
 
     for (entry = indices; entry->if_index; entry++)
     {
@@ -301,6 +342,14 @@ static unsigned int update_if_table( void )
     }
 
     if_freenameindex( indices );
+
+    if (!have_ethernet_iface && fake_ethernet_adapter() && (if_entry = add_entry( 0xdeadbeef, "eth0faked" )))
+    {
+        if_entry->if_type = if_entry->if_luid.Info.IfType = MIB_IF_TYPE_ETHERNET;
+        have_ethernet_iface = TRUE;
+        ++append_count;
+    }
+
     return append_count;
 }
 
