@@ -44,11 +44,12 @@ struct media_stream
     LONG token_queue_count;
     LONG token_queue_cap;
 
-    CRITICAL_SECTION cs;
-
     DWORD stream_id;
     BOOL active;
     BOOL eos;
+
+    DWORD busy;
+    CONDITION_VARIABLE cond;
 };
 
 enum source_async_op
@@ -538,17 +539,12 @@ static void wait_on_sample(struct media_stream *stream, IUnknown *token)
 
     TRACE("%p, %p\n", stream, token);
 
-    EnterCriticalSection(&stream->cs);
-
-    if (!stream->wg_stream)
-    {
-        LeaveCriticalSection(&stream->cs);
-        return;
-    }
-
+    stream->busy = TRUE;
     LeaveCriticalSection(&source->cs);
     ret = wg_parser_stream_get_buffer(source->wg_parser, stream->wg_stream, &buffer);
     EnterCriticalSection(&source->cs);
+    stream->busy = FALSE;
+    WakeConditionVariable(&stream->cond);
 
     if (source->state == SOURCE_SHUTDOWN)
         WARN("media source has been shutdown, returning\n");
@@ -560,8 +556,6 @@ static void wait_on_sample(struct media_stream *stream, IUnknown *token)
         IMFMediaEventQueue_QueueEventParamVar(stream->event_queue, MEEndOfStream, &GUID_NULL, S_OK, &empty_var);
         dispatch_end_of_presentation(source);
     }
-
-    LeaveCriticalSection(&stream->cs);
 }
 
 static HRESULT WINAPI source_async_commands_Invoke(IMFAsyncCallback *iface, IMFAsyncResult *result)
@@ -721,8 +715,6 @@ static ULONG WINAPI media_stream_Release(IMFMediaStream *iface)
         IMFStreamDescriptor_Release(stream->descriptor);
         IMFMediaEventQueue_Release(stream->event_queue);
         flush_token_queue(stream, FALSE);
-        stream->cs.DebugInfo->Spare[0] = 0;
-        DeleteCriticalSection(&stream->cs);
         free(stream);
     }
 
@@ -887,9 +879,6 @@ static HRESULT media_stream_create(IMFMediaSource *source, DWORD id,
     object->eos = FALSE;
     object->wg_stream = wg_parser_get_stream(wg_parser, id);
 
-    InitializeCriticalSection(&object->cs);
-    object->cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": cs");
-
     TRACE("Created stream object %p.\n", object);
 
     *out = object;
@@ -1247,6 +1236,7 @@ static ULONG WINAPI media_source_Release(IMFMediaSource *iface)
     if (!ref)
     {
         IMFMediaSource_Shutdown(iface);
+        MFUnlockWorkQueue(source->async_commands_queue);
         IMFPresentationDescriptor_Release(source->pres_desc);
         IMFMediaEventQueue_Release(source->event_queue);
         IMFByteStream_Release(source->byte_stream);
@@ -1427,10 +1417,8 @@ static HRESULT WINAPI media_source_Shutdown(IMFMediaSource *iface)
     {
         struct media_stream *stream = source->streams[i];
         wg_parser_stream_disable(stream->wg_stream);
-
-        EnterCriticalSection(&stream->cs);
-        stream->wg_stream = NULL;
-        LeaveCriticalSection(&stream->cs);
+        while (stream->busy)
+            SleepConditionVariableCS(&stream->cond, &source->cs, INFINITE);
     }
 
     wg_parser_disconnect(source->wg_parser);
@@ -1450,8 +1438,6 @@ static HRESULT WINAPI media_source_Shutdown(IMFMediaSource *iface)
     }
     free(source->streams);
 
-    MFUnlockWorkQueue(source->async_commands_queue);
-
     LeaveCriticalSection(&source->cs);
 
     return S_OK;
