@@ -248,6 +248,8 @@ static void modes_append( DEVMODEW *modes, UINT *mode_count, UINT *resolutions,
         if (modes[i].dmBitsPerPel != mode->dmBitsPerPel) continue;
         if (modes[i].dmDisplayFrequency != mode->dmDisplayFrequency) continue;
         if (modes[i].dmDisplayOrientation != mode->dmDisplayOrientation) continue;
+        if ((mode->dmFields & DM_DISPLAYFIXEDOUTPUT) != (modes[i].dmFields & DM_DISPLAYFIXEDOUTPUT)) continue;
+        if (mode->dmFields & DM_DISPLAYFIXEDOUTPUT && modes[i].dmDisplayFixedOutput != mode->dmDisplayFixedOutput) continue;
         return; /* The exact mode is already added, nothing to do */
     }
 
@@ -259,7 +261,7 @@ static void modes_append( DEVMODEW *modes, UINT *mode_count, UINT *resolutions,
     }
 
     mode->dmFields = DM_DISPLAYORIENTATION | DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT |
-                     DM_DISPLAYFLAGS | DM_DISPLAYFREQUENCY;
+                     DM_DISPLAYFLAGS | DM_DISPLAYFREQUENCY | (mode->dmFields & DM_DISPLAYFIXEDOUTPUT);
     mode->dmSize = sizeof(DEVMODEW);
     mode->dmDriverExtra = 0;
     mode->dmDisplayFlags = 0;
@@ -274,7 +276,7 @@ static void monitor_get_modes( struct fs_monitor *monitor, DEVMODEW **modes, UIN
 {
     UINT i, j, max_count, real_mode_count, resolutions = 0;
     DEVMODEW *real_modes, *real_mode, mode_host = {0};
-    BOOL additional_modes = FALSE;
+    BOOL additional_modes = FALSE, center_modes = FALSE, landscape;
     const char *env;
 
     *mode_count = 0;
@@ -284,13 +286,25 @@ static void monitor_get_modes( struct fs_monitor *monitor, DEVMODEW **modes, UIN
     /* Fullscreen hack doesn't support changing display orientations */
     if (!real_settings_handler.get_modes( monitor->adapter_id, 0, &real_modes, &real_mode_count )) return;
 
+    if ((env = getenv( "WINE_CENTER_DISPLAY_MODES" )))
+        center_modes = (env[0] != '0');
+    else if ((env = getenv( "SteamAppId" )))
+        center_modes = !strcmp( env, "359870" );
+
     max_count = ARRAY_SIZE(fs_monitor_sizes) * DEPTH_COUNT + real_mode_count;
+    if (center_modes) max_count += ARRAY_SIZE(fs_monitor_sizes) + real_mode_count;
+
     if (!(*modes = calloc( max_count, sizeof(DEVMODEW) )))
     {
         real_settings_handler.free_modes( real_modes );
         return;
     }
 
+    /* Check the ratio of dmPelsWidth to dmPelsHeight to determine whether the host is currently in
+     * portrait or landscape orientation. DMDO_DEFAULT is the natural orientation of the device,
+     * which isn't necessarily a landscape mode */
+    landscape = mode_host.dmPelsWidth >= mode_host.dmPelsHeight;
+
     /* Add the current mode early, in case we have to limit */
     modes_append( *modes, mode_count, &resolutions, &mode_host );
 
@@ -306,8 +320,7 @@ static void monitor_get_modes( struct fs_monitor *monitor, DEVMODEW **modes, UIN
 
         if (!additional_modes && fs_monitor_sizes[i].additional) continue;
 
-        if (mode_host.dmDisplayOrientation == DMDO_DEFAULT ||
-            mode_host.dmDisplayOrientation == DMDO_180)
+        if (landscape)
         {
             mode.dmPelsWidth = fs_monitor_sizes[i].size.cx;
             mode.dmPelsHeight = fs_monitor_sizes[i].size.cy;
@@ -328,6 +341,13 @@ static void monitor_get_modes( struct fs_monitor *monitor, DEVMODEW **modes, UIN
             mode.dmDisplayFrequency = 60;
             modes_append( *modes, mode_count, &resolutions, &mode );
         }
+
+        if (center_modes && mode.dmPelsWidth != mode_host.dmPelsWidth && mode.dmPelsHeight != mode_host.dmPelsHeight)
+        {
+            mode.dmFields |= DM_DISPLAYFIXEDOUTPUT;
+            mode.dmDisplayFixedOutput = DMDFO_CENTER;
+            modes_append( *modes, mode_count, &resolutions, &mode );
+        }
     }
 
     for (i = 0, real_mode = real_modes; i < real_mode_count; ++i)
@@ -336,8 +356,17 @@ static void monitor_get_modes( struct fs_monitor *monitor, DEVMODEW **modes, UIN
 
         /* Don't report modes that are larger than the current mode */
         if (mode.dmPelsWidth <= mode_host.dmPelsWidth && mode.dmPelsHeight <= mode_host.dmPelsHeight)
+        {
             modes_append( *modes, mode_count, &resolutions, &mode );
 
+            if (center_modes && mode.dmPelsWidth != mode_host.dmPelsWidth && mode.dmPelsHeight != mode_host.dmPelsHeight)
+            {
+                mode.dmFields |= DM_DISPLAYFIXEDOUTPUT;
+                mode.dmDisplayFixedOutput = DMDFO_CENTER;
+                modes_append( *modes, mode_count, &resolutions, &mode );
+            }
+        }
+
         real_mode = NEXT_DEVMODEW(real_mode);
     }
 
@@ -403,7 +432,6 @@ static BOOL fs_get_current_mode( ULONG_PTR adapter_id, DEVMODEW *mode )
 
 static LONG fs_set_current_mode( ULONG_PTR adapter_id, const DEVMODEW *user_mode )
 {
-    WCHAR device_name[CCHDEVICENAME];
     struct fs_monitor *fs_monitor;
     DEVMODEW real_mode;
     double scale;
@@ -433,7 +461,7 @@ static LONG fs_set_current_mode( ULONG_PTR adapter_id, const DEVMODEW *user_mode
 
     fs_monitor->user_mode = *user_mode;
     fs_monitor->real_mode = real_mode;
-    lstrcpyW( fs_monitor->user_mode.dmDeviceName, device_name );
+    lstrcpyW( fs_monitor->user_mode.dmDeviceName, L"fshack" );
 
     if (is_detached_mode( user_mode ))
     {
