@@ -26,6 +26,7 @@
 #include "ws2tcpip.h"
 #include "winhttp.h"
 #include "schannel.h"
+#include "winternl.h"
 
 #include "wine/debug.h"
 #include "winhttp_private.h"
@@ -56,15 +57,16 @@ BOOL netconn_wait_overlapped_result( struct netconn *conn, WSAOVERLAPPED *ovr, D
     OVERLAPPED *completion_ovr;
     ULONG_PTR key;
 
-    if (!GetQueuedCompletionStatus( conn->port, len, &key, &completion_ovr, INFINITE ))
+    while (1)
     {
-        WARN( "GetQueuedCompletionStatus failed, err %lu.\n", GetLastError() );
-        return FALSE;
-    }
-    if ((key != conn->socket && conn->socket != -1) || completion_ovr != (OVERLAPPED *)ovr)
-    {
-        ERR( "Unexpected completion key %Ix, overlapped %p.\n", key, completion_ovr );
-        return FALSE;
+        if (!GetQueuedCompletionStatus( conn->port, len, &key, &completion_ovr, INFINITE ))
+        {
+            WARN( "GetQueuedCompletionStatus failed, err %lu.\n", GetLastError() );
+            return FALSE;
+        }
+        if (completion_ovr == (OVERLAPPED *)ovr && (key == conn->socket || conn->socket == -1))
+            break;
+        ERR( "Unexpected completion key %Ix, completion ovr %p, ovr %p.\n", key, completion_ovr, ovr );
     }
     return TRUE;
 }
@@ -224,6 +226,8 @@ DWORD netconn_create( struct hostdata *host, const struct sockaddr_storage *sock
         free( conn );
         return ret;
     }
+    if (!SetFileCompletionNotificationModes( (HANDLE)(UINT_PTR)conn->socket, FILE_SKIP_COMPLETION_PORT_ON_SUCCESS ))
+        ERR( "SetFileCompletionNotificationModes failed.\n" );
 
     switch (conn->sockaddr.ss_family)
     {
